const { ccclass, property } = cc._decorator;

@ccclass
export default class MaskLight extends cc.Component {
    @property(cc.Sprite)
    maskSprite: cc.Sprite = null;

    pointLightMap: any = {};
    mat: cc.Material = null;
    widthHalf: number = cc.winSize.width / 2;
    heightHalf: number = cc.winSize.height / 2;

    onLoad() {
        this.mat = this.maskSprite.getMaterial(0);
        this.mat.setProperty("wh_ratio", this.widthHalf / this.heightHalf)

        this.imAddPointLight(-100, 300)
        this.imAddPointLight(-100, 100, 100)
        this.imAddPointLight(200, 200, 200)
        this.imAddPointLight(-300, 0, 200, 0.5)
    }

    imAddPointLight(x, y, radius = 375, value = 1) {
        let idx = 0;
        for (let i = 1; i <= 5; i++) {
            if (this.pointLightMap[i] == undefined) {
                let v4 = new cc.Vec4(x, y, 0, value);
                this.pointLightMap[i] = v4;
                idx = i;
                break;
            } else if (this.pointLightMap[i].w <= 0) {
                idx = i;
                break;
            }
        }
        if (idx == 0) return;
        this.pointLightMap[idx].x = x / this.widthHalf;
        this.pointLightMap[idx].y = y / this.heightHalf;
        this.pointLightMap[idx].z = radius / cc.winSize.height;
        this.pointLightMap[idx].w = value;
        this.mat.setProperty("center" + idx, this.pointLightMap[idx]);
        return idx;
    }

    //关闭某个点灯光
    imClosePointLight(idx) {
        if (this.pointLightMap[idx] && this.pointLightMap[idx].w > 0) {
            this.pointLightMap[idx].w = -0.1;
            this.mat.setProperty("center" + idx, this.pointLightMap[idx]);
        }
    }
}